@@ -1,14 +1,21 @@
 /*
  * container_binary_array.c
- * $Id: container_binary_array.c 20119 2011-03-12 06:59:56Z rstory $
  *
  * see comments in header file.
  *
+ * Portions of this file are subject to the following copyright(s).  See
+ * the Net-SNMP's COPYING file for more details and other copyrights
+ * that may apply:
+ *
+ * Portions of this file are copyrighted by:
+ * Copyright (c) 2016 VMware, Inc. All rights reserved.
+ * Use is subject to license terms specified in the COPYING file
+ * distributed with the Net-SNMP package.
  */
 
 #include <net-snmp/net-snmp-config.h>
 
-#if HAVE_IO_H
+#ifdef HAVE_IO_H
 #include <io.h>
 #endif
 #include <stdio.h>
@@ -32,13 +39,12 @@
 #include <net-snmp/library/container_binary_array.h>
 #include <net-snmp/library/tools.h>
 #include <net-snmp/library/snmp_assert.h>
+#include "factory.h"
 
 typedef struct binary_array_table_s {
     size_t                     max_size;   /* Size of the current data table */
     size_t                     count;      /* Index of the next free entry */
-    u_int                      flags;      /* flags */
     int                        dirty;
-    int                        data_size;  /* Size of an individual entry */
     void                     **data;       /* The table itself */
 } binary_array_table;
 
@@ -50,48 +56,6 @@ typedef struct binary_array_iterator_s {
 
 static netsnmp_iterator *_ba_iterator_get(netsnmp_container *c);
 
-/**********************************************************************
- *
- * 
- *
- */
-static void
-array_qsort(void **data, int first, int last, netsnmp_container_compare *f)
-{
-    int i, j;
-    void *mid, *tmp;
-    
-    i = first;
-    j = last;
-    mid = data[(first+last)/2];
-    
-    do {
-        while ( ((*f)(data[i], mid) < 0) && (i < last))
-            ++i;
-        while ( ((*f)(mid, data[j]) < 0) && (j > first))
-            --j;
-
-        if(i < j) {
-            tmp = data[i];
-            data[i] = data[j];
-            data[j] = tmp;
-            ++i;
-            --j;
-        }
-        else if (i == j) {
-            ++i;
-            --j;
-            break;
-        }
-    } while(i <= j);
-
-    if (j > first)
-        array_qsort(data, first, j, f);
-    
-    if (i < last)
-        array_qsort(data, i, last, f);
-}
-
 static int
 Sort_Array(netsnmp_container *c)
 {
@@ -99,17 +63,19 @@ Sort_Array(netsnmp_container *c)
     netsnmp_assert(t!=NULL);
     netsnmp_assert(c->compare!=NULL);
 
-    if (t->flags & CONTAINER_KEY_UNSORTED)
+    if (c->flags & CONTAINER_KEY_UNSORTED)
         return 0;
 
     if (t->dirty) {
         /*
          * Sort the table 
          */
-        if (t->count > 1)
-            array_qsort(t->data, 0, t->count - 1, c->compare);
+        qsort(t->data, t->count, sizeof(void *), c->compare);
         t->dirty = 0;
 
+        /*
+         * no way to know if it actually changed... just assume so.
+         */
         ++c->sync;
     }
 
@@ -117,56 +83,97 @@ Sort_Array(netsnmp_container *c)
 }
 
 static int
-binary_search(const void *val, netsnmp_container *c, int exact)
+linear_search(const void *val, netsnmp_container *c)
+{
+    binary_array_table *t = (binary_array_table*)c->container_data;
+    size_t             pos = 0;
+
+    if (!t->count)
+        return -1;
+
+    if (! (c->flags & CONTAINER_KEY_UNSORTED)) {
+        snmp_log(LOG_ERR, "linear search on sorted container %s?!?\n",
+                 c->container_name);
+        return -1;
+    }
+
+    for (; pos < t->count; ++pos) {
+        if (c->compare(t->data[pos], val) == 0)
+            return pos;
+    }
+
+    return -1;
+}
+
+static int
+binary_search(const void *val, netsnmp_container *c, int exact, size_t *next)
 {
     binary_array_table *t = (binary_array_table*)c->container_data;
     size_t             len = t->count;
     size_t             half;
-    size_t             middle = 0;
     size_t             first = 0;
-    int                result = 0;
+    size_t             middle = 0; /* init not needed; keeps compiler happy */
+    int                result = 0; /* init not needed; keeps compiler happy */
 
-    if (!len)
+    if (!len) {
+        if (NULL != next)
+            *next = 0;
         return -1;
+    }
+
+    if (c->flags & CONTAINER_KEY_UNSORTED) {
+        if (!exact) {
+            snmp_log(LOG_ERR, "non-exact search on unsorted container %s?!?\n",
+                     c->container_name);
+            return -1;
+        }
+        return linear_search(val, c);
+    }
 
     if (t->dirty)
         Sort_Array(c);
 
     while (len > 0) {
         half = len >> 1;
-        middle = first;
-        middle += half;
-        if ((result =
-             c->compare(t->data[middle], val)) < 0) {
-            first = middle;
-            ++first;
+        middle = first + half;
+        if ((result = c->compare(t->data[middle], val)) < 0) {
+            first = middle + 1;
             len = len - half - 1;
+        } else if (result == 0) {
+            first = middle;
+            break;
         } else {
-            if(result == 0) {
-                first = middle;
-                break;
-            }
             len = half;
         }
     }
 
-    if (first >= t->count)
+    if (first >= t->count) {
+        if (exact && NULL != next)
+            *next = t->count;
         return -1;
+    }
 
-    if(first != middle) {
+    if (first != middle) {
         /* last compare wasn't against first, so get actual result */
         result = c->compare(t->data[first], val);
     }
 
     if(result == 0) {
-        if (!exact) {
-            if (++first == t->count)
-               first = -1;
-        }
-    }
-    else {
-        if(exact)
+        if (exact && NULL != next)
+            *next = first+1;
+        else if (!exact && ++first == t->count) {
+            if (NULL != next)
+                *next = first;
             first = -1;
+        }
+    } else if(exact) {
+        if (NULL != next) {
+            if (result > 0)
+                *next = first;
+            else
+                *next = t->count;
+        }
+        first = -1;
     }
 
     return first;
@@ -184,7 +191,6 @@ netsnmp_binary_array_initialize(void)
     t->max_size = 0;
     t->count = 0;
     t->dirty = 0;
-    t->data_size = sizeof(void*);
     t->data = NULL;
 
     return t;
@@ -194,22 +200,40 @@ void
 netsnmp_binary_array_release(netsnmp_container *c)
 {
     binary_array_table *t = (binary_array_table*)c->container_data;
-    if (t->data != NULL) {
-	SNMP_FREE(t->data);
-    }
+    SNMP_FREE(t->data);
     SNMP_FREE(t);
     SNMP_FREE(c);
 }
 
+/**
+ * Set or test the options of a binary array container.
+ * @param c: Container.
+ * @param set: Set (1) or test (0).
+ * @param flags: Zero or more CONTAINER_KEY_* flags.
+ */
 int
 netsnmp_binary_array_options_set(netsnmp_container *c, int set, u_int flags)
 {
-    binary_array_table *t = (binary_array_table*)c->container_data;
-    if (set)
-        t->flags = flags;
-    else
-        return ((t->flags & flags) == flags);
-    return flags;
+#define BA_FLAGS (CONTAINER_KEY_ALLOW_DUPLICATES|CONTAINER_KEY_UNSORTED)
+
+    if (set) {
+        if ((flags & BA_FLAGS) == flags) {
+            /** if turning off unsorted, do sort */
+            int sort = ((c->flags & CONTAINER_KEY_UNSORTED) &&
+                        ! (flags & CONTAINER_KEY_UNSORTED));
+            c->flags = flags;
+            if (sort) {
+                binary_array_table *t = (binary_array_table*)c->container_data;
+                t->dirty = 1; /* force sort */
+                Sort_Array(c);
+            }
+            return flags;
+        } else {
+            return -1; /* unsupported flag */
+        }
+    } else {
+        return ((c->flags & flags) == flags);
+    }
 }
 
 NETSNMP_STATIC_INLINE size_t
@@ -232,7 +256,7 @@ netsnmp_binary_array_get(netsnmp_container *c, const void *key, int exact)
      * if there is no data, return NULL;
      */
     if (!t->count)
-        return 0;
+        return NULL;
 
     /*
      * if the table is dirty, sort it.
@@ -244,18 +268,116 @@ netsnmp_binary_array_get(netsnmp_container *c, const void *key, int exact)
      * if there is a key, search. Otherwise default is 0;
      */
     if (key) {
-        if ((index = binary_search(key, c, exact)) == -1)
-            return 0;
+        if ((index = binary_search(key, c, exact, NULL)) == -1)
+            return NULL;
+        if (!exact &&
+            c->flags & CONTAINER_KEY_ALLOW_DUPLICATES) {
+            int result;
+
+            /*
+             * If duplicates are allowed, we have to be extra
+             * sure that we didn't just increment to a duplicate,
+             * thus causing a getnext loop.
+             */
+            result = c->compare(t->data[index], key);
+            while (result == 0) {
+		DEBUGMSGTL(("container","skipping duplicate key in %s\n",
+					    c->container_name));
+                if (++index == t->count)
+                   return NULL;
+                result = c->compare(t->data[index], key);
+            }
+        }
     }
 
     return t->data[index];
 }
 
+static int
+netsnmp_binary_array_get_at(netsnmp_container *c, size_t pos, void **entry)
+{
+    binary_array_table *t = (binary_array_table*)c->container_data;
+
+    /*
+     * if there is no data, return NULL;
+     */
+    if (!t->count || pos >= t->count || NULL == entry)
+        return -1;
+
+    *entry = t->data[pos];
+
+    return 0;
+}
+
+/**
+ * Returns 1 if and only if the elements in @c are sorted in ascending order.
+ *
+ * To do: stop calling this function after
+ * https://github.com/net-snmp/net-snmp/issues/107 and
+ * https://github.com/net-snmp/net-snmp/issues/293 have been fixed.
+ */
+static int _ba_is_sorted(const netsnmp_container *c)
+{
+    /*
+     * The code below has been commented out because it negatively affects
+     * performance.
+     */
+#if 0
+    const binary_array_table *t = c->container_data;
+    int i;
+
+    for (i = 0; i + 1 < t->count; ++i)
+        if (c->compare(t->data[i], t->data[i + 1]) > 0)
+            return 0;
+#endif
+
+    return 1;
+}
+
+int
+netsnmp_binary_array_remove_at(netsnmp_container *c, size_t index, void **save)
+{
+    binary_array_table *t = (binary_array_table*)c->container_data;
+
+    if (save)
+        *save = NULL;
+    
+    /*
+     * if there is no data, return NULL;
+     */
+    if (!t->count)
+        return -1;
+
+    /*
+     * find old data and save it, if ptr provided
+     */
+    if (save)
+        *save = t->data[index];
+
+    /*
+     * if entry was last item, just decrement count
+     */
+    --t->count;
+    if (index != t->count) {
+        /*
+         * otherwise, shift array down
+         */
+        memmove(&t->data[index], &t->data[index+1],
+                sizeof(void*) * (t->count - index));
+
+        ++c->sync;
+    }
+
+    netsnmp_assert(t->dirty || _ba_is_sorted(c));
+
+    return 0;
+}
+
 int
 netsnmp_binary_array_remove(netsnmp_container *c, const void *key, void **save)
 {
     binary_array_table *t = (binary_array_table*)c->container_data;
-    size_t             index = 0;
+    int                index = 0;
 
     if (save)
         *save = NULL;
@@ -275,27 +397,10 @@ netsnmp_binary_array_remove(netsnmp_container *c, const void *key, void **save)
     /*
      * search
      */
-    if ((index = binary_search(key, c, 1)) == -1)
+    if ((index = binary_search(key, c, 1, NULL)) == -1)
         return -1;
 
-    /*
-     * find old data and save it, if ptr provided
-     */
-    if (save)
-        *save = t->data[index];
-
-    /*
-     * if entry was last item, just decrement count
-     */
-    --t->count;
-    if (index != t->count) {
-        /*
-         * otherwise, shift array down
-         */
-        memmove(&t->data[index], &t->data[index+1], t->data_size * (t->count - index));
-    }
-
-    return 0;
+    return netsnmp_binary_array_remove_at(c, (size_t)index, save);
 }
 
 NETSNMP_STATIC_INLINE void
@@ -332,56 +437,130 @@ netsnmp_binary_array_clear(netsnmp_container *c,
     ++c->sync;
 }
 
-NETSNMP_STATIC_INLINE int
-netsnmp_binary_array_insert(netsnmp_container *c, const void *entry)
+static int
+_ba_resize_check(binary_array_table *t)
+{
+    size_t new_max;
+    void ** new_data;
+    if (t->max_size > t->count)
+        return 0; /* resize not needed */
+
+    /*
+     * Table is full, so extend it to double the size, or use 10 elements
+     * if it is empty.
+     */
+    new_max = t->max_size > 0 ? 2 * t->max_size : 10;
+    new_data = (void**) realloc(t->data, new_max * sizeof(void*));
+    if (new_data == NULL) {
+        snmp_log(LOG_ERR, "malloc failed in _ba_resize_check\n");
+        return -1; /* error */
+    }
+
+    memset(new_data + t->max_size, 0x0,
+           (new_max - t->max_size) * sizeof(void*));
+
+    t->data = new_data;
+    t->max_size = new_max;
+
+    return 1; /* resized */
+}
+
+static int
+netsnmp_binary_array_insert_before(netsnmp_container *c, size_t index,
+                                   const void *entry, int dirty)
 {
     binary_array_table *t = (binary_array_table*)c->container_data;
-    int             new_max, new_size;
-    char           *new_data;   /* Used for * a) extending the data table
-                                 * * b) the next entry to use */
-    /*
-     * check for duplicates
-     */
-    if (! (t->flags & CONTAINER_KEY_ALLOW_DUPLICATES)) {
-        new_data = netsnmp_binary_array_get(c, entry, 1);
-        if (NULL != new_data) {
-            DEBUGMSGTL(("container","not inserting duplicate key\n"));
-            return -1;
-        }
-    }
-    
-    /*
-     * check if we need to resize the array
-     */
-    if (t->max_size <= t->count) {
-        /*
-         * Table is full, so extend it to double the size
-         */
-        new_max = 2 * t->max_size;
-        if (new_max == 0)
-            new_max = 10;       /* Start with 10 entries */
 
-        new_size = new_max * t->data_size;
-        new_data = (char *) realloc(t->data, new_size);
-        if (new_data == NULL)
-            return -1;
-        else {
-            int old_size = t->max_size * t->data_size;
-            int count = new_size - old_size;
-            memset(&new_data[old_size], 0x0, count);
-        }
-        t->data = (void**)new_data;
-        t->max_size = new_max;
+    if (NULL == entry)
+        return -1;
+
+    if (index > t->count) {
+        DEBUGMSGTL(("container:insert:before", "index out of range\n"));
+        return -1;
     }
 
+     /*
+      * check if we need to resize the array
+      */
+    _ba_resize_check(t);
+
+    netsnmp_assert(t->count < t->max_size);
+
+    /*
+     * shift array
+     */
+    memmove(&t->data[index+1], &t->data[index],
+            sizeof(void*) * (t->count - index));
+
     /*
      * Insert the new entry into the data array
      */
-    t->data[t->count++] = (void *)entry;
-    t->dirty = 1;
+    t->data[index] = NETSNMP_REMOVE_CONST(void *, entry);
+    ++t->count;
+
+    netsnmp_assert(index < t->count);
+    netsnmp_assert(t->count <= t->max_size);
+
+    if (dirty)
+        t->dirty = 1;
+
+    netsnmp_assert(t->dirty || _ba_is_sorted(c));
+
+    ++c->sync;
+
     return 0;
 }
 
+NETSNMP_STATIC_INLINE int
+netsnmp_binary_array_insert(netsnmp_container *c, const void *const_entry)
+{
+    binary_array_table *t = (binary_array_table*)c->container_data;
+    const int duplicates_allowed = c->flags & CONTAINER_KEY_ALLOW_DUPLICATES;
+    const int sorted = !(c->flags & CONTAINER_KEY_UNSORTED);
+    int             i = -2;
+    size_t          next, pos;
+    void           *entry = NETSNMP_REMOVE_CONST(void *, const_entry);
+
+    if (NULL == entry)
+        return -1;
+
+    /*
+     * check key if we have at least 1 item and duplicates aren't allowed
+     */
+    if (!duplicates_allowed && t->count) {
+        i = binary_search(entry, c, 1, &next);
+        if (i >= 0) {
+            DEBUGMSGTL(("container","not inserting duplicate key\n"));
+            return -1;
+        }
+    }
+ 
+    /*
+     * if unsorted, just add at the end
+     */
+    if (!sorted) {
+        pos = t->count;
+    } else {
+        /** if we haven't searched for key yet, do it now */
+        if (-2 == i) {
+            if (0 == t->count) {
+                next = 0;
+                i = -1;
+            } else {
+                i = binary_search(entry, c, 1, &next);
+            }
+        }
+
+        pos = next;
+        /* if key found, advance past any duplicates */
+        if (duplicates_allowed && i >= 0)
+            while (pos < t->count && c->compare(t->data[pos], entry) == 0)
+                ++pos;
+    }
+
+    return netsnmp_binary_array_insert_before(c, pos, entry, !sorted);
+}
+
 /**********************************************************************
  *
  * Special case support for subsets
@@ -405,11 +584,9 @@ binary_search_for_start(netsnmp_index *val, netsnmp_container *c)
 
     while (len > 0) {
         half = len >> 1;
-        middle = first;
-        middle += half;
+        middle = first + half;
         if ((result = c->ncompare(t->data[middle], val)) < 0) {
-            first = middle;
-            ++first;
+            first = middle + 1;
             len = len - half - 1;
         } else
             len = half;
@@ -425,7 +602,7 @@ binary_search_for_start(netsnmp_index *val, netsnmp_container *c)
 void          **
 netsnmp_binary_array_get_subset(netsnmp_container *c, void *key, int *len)
 {
-    binary_array_table *t = (binary_array_table*)c->container_data;
+    binary_array_table *t;
     void          **subset;
     int             start, end;
     size_t          i;
@@ -433,8 +610,13 @@ netsnmp_binary_array_get_subset(netsnmp_container *c, void *key, int *len)
     /*
      * if there is no data, return NULL;
      */
-    if (!t->count || !key)
-        return 0;
+    if (!c || !key || !len)
+        return NULL;
+
+    t = (binary_array_table*)c->container_data;
+    netsnmp_assert(c->ncompare);
+    if (!t->count || !c->ncompare)
+        return NULL;
 
     /*
      * if the table is dirty, sort it.
@@ -447,7 +629,7 @@ netsnmp_binary_array_get_subset(netsnmp_container *c, void *key, int *len)
      */
     start = end = binary_search_for_start((netsnmp_index *)key, c);
     if (start == -1)
-        return 0;
+        return NULL;
 
     for (i = start + 1; i < t->count; ++i) {
         if (0 != c->ncompare(t->data[i], key))
@@ -456,9 +638,12 @@ netsnmp_binary_array_get_subset(netsnmp_container *c, void *key, int *len)
     }
 
     *len = end - start + 1;
-    subset = (void **)malloc((*len) * t->data_size);
+    if (*len <= 0)
+        return NULL;
+
+    subset = (void **)malloc((*len) * sizeof(void*));
     if (subset)
-        memcpy(subset, &t->data[start], t->data_size * (*len));
+        memcpy(subset, &t->data[start], sizeof(void*) * (*len));
 
     return subset;
 }
@@ -486,6 +671,13 @@ _ba_insert(netsnmp_container *container, const void *data)
     return netsnmp_binary_array_insert(container, data);
 }
 
+static int
+_ba_insert_before(netsnmp_container *container, size_t index, void *data)
+{
+    /** don't trust users direct-acces inserts, mark array dirty */
+    return netsnmp_binary_array_insert_before(container, index, data, 1);
+}
+
 static int
 _ba_remove(netsnmp_container *container, const void *data)
 {
@@ -527,13 +719,12 @@ _ba_get_subset(netsnmp_container *container, void *data)
     int len;
 
     rtn = netsnmp_binary_array_get_subset(container, data, &len);
-    if ((NULL==rtn) || (len <=0))
+    if (NULL==rtn)
         return NULL;
     
     va = SNMP_MALLOC_TYPEDEF(netsnmp_void_array);
-    if (NULL==va)
-    {
-        free (rtn);
+    if (va == NULL) {
+        free(rtn);
         return NULL;
     }
     
@@ -543,6 +734,60 @@ _ba_get_subset(netsnmp_container *container, void *data)
     return va;
 }
 
+static int _ba_options(netsnmp_container *c, int set, u_int flags)
+{
+    return netsnmp_binary_array_options_set(c, set, flags);
+}
+
+static netsnmp_container *
+_ba_duplicate(netsnmp_container *c, void *ctx, u_int flags)
+{
+    netsnmp_container *dup;
+    binary_array_table *dupt, *t;
+
+    if (flags) {
+        snmp_log(LOG_ERR, "binary arry duplicate does not supprt flags yet\n");
+        return NULL;
+    }
+
+    dup = netsnmp_container_get_binary_array();
+    if (NULL == dup) {
+        snmp_log(LOG_ERR," no memory for binary array duplicate\n");
+        return NULL;
+    }
+    /*
+     * deal with container stuff
+     */
+    if (netsnmp_container_data_dup(dup, c) != 0) {
+        netsnmp_binary_array_release(dup);
+        return NULL;
+    }
+
+    /*
+     * deal with data
+     */
+    dupt = (binary_array_table*)dup->container_data;
+    t = (binary_array_table*)c->container_data;
+
+    dupt->max_size = t->max_size;
+    dupt->count = t->count;
+    dupt->dirty = t->dirty;
+
+    /*
+     * shallow copy
+     */
+    dupt->data = (void**) malloc(dupt->max_size * sizeof(void*));
+    if (NULL == dupt->data) {
+        snmp_log(LOG_ERR, "no memory for binary array duplicate\n");
+        netsnmp_binary_array_release(dup);
+        return NULL;
+    }
+
+    memcpy(dupt->data, t->data, dupt->max_size * sizeof(void*));
+
+    return dup;
+}
+
 netsnmp_container *
 netsnmp_container_get_binary_array(void)
 {
@@ -556,19 +801,23 @@ netsnmp_container_get_binary_array(void)
     }
 
     c->container_data = netsnmp_binary_array_initialize();
-        
-    c->get_size = _ba_size;
-    c->init = NULL;
-    c->cfree = _ba_free;
-    c->insert = _ba_insert;
-    c->remove = _ba_remove;
-    c->find = _ba_find;
+
+    /*
+     * NOTE: CHANGES HERE MUST BE DUPLICATED IN duplicate AS WELL!!
+     */
+    netsnmp_init_container(c, NULL, _ba_free, _ba_size, NULL, _ba_insert,
+                           _ba_remove, _ba_find);
     c->find_next = _ba_find_next;
     c->get_subset = _ba_get_subset;
     c->get_iterator = _ba_iterator_get;
     c->for_each = _ba_for_each;
     c->clear = _ba_clear;
-        
+    c->options = _ba_options;
+    c->duplicate = _ba_duplicate;
+    c->get_at = netsnmp_binary_array_get_at;
+    c->remove_at = netsnmp_binary_array_remove_at;
+    c->insert_before = _ba_insert_before;
+
     return c;
 }
 
@@ -576,7 +825,6 @@ netsnmp_factory *
 netsnmp_container_get_binary_array_factory(void)
 {
     static netsnmp_factory f = { "binary_array",
-                                 (netsnmp_factory_produce_f*)
                                  netsnmp_container_get_binary_array };
     
     return &f;
@@ -679,6 +927,25 @@ _ba_iterator_last(binary_array_iterator *it)
     return _ba_iterator_position(it, t->count - 1 );
 }
 
+static int
+_ba_iterator_remove(binary_array_iterator *it)
+{
+    binary_array_table* t = _ba_it2cont(it);
+    if(NULL == t) {
+        netsnmp_assert(NULL != t);
+        return -1;
+    }
+
+    /*
+     * since this iterator was used for the remove, keep it in sync with
+     * the container. Also, back up one so that next will be the position
+     * that was just removed.
+     */
+    ++it->base.sync;
+    return netsnmp_binary_array_remove_at(it->base.container, it->pos--, NULL);
+
+}
+
 static int
 _ba_iterator_reset(binary_array_iterator *it)
 {
@@ -728,6 +995,7 @@ _ba_iterator_get(netsnmp_container *c)
     it->base.next = (netsnmp_iterator_rtn*)_ba_iterator_next;
     it->base.curr = (netsnmp_iterator_rtn*)_ba_iterator_curr;
     it->base.last = (netsnmp_iterator_rtn*)_ba_iterator_last;
+    it->base.remove = (netsnmp_iterator_rc*)_ba_iterator_remove;
     it->base.reset = (netsnmp_iterator_rc*)_ba_iterator_reset;
     it->base.release = (netsnmp_iterator_rc*)_ba_iterator_release;
 
